@@ -88,7 +88,7 @@ public:
     ~QQuickViewPrivate();
 
     void execute();
-    void itemGeometryChanged(QQuickItem *item, const QRectF &newGeometry, const QRectF &oldGeometry) Q_DECL_OVERRIDE;
+    void itemGeometryChanged(QQuickItem *item, QQuickGeometryChange change, const QRectF &) Q_DECL_OVERRIDE;
     void initResize();
     void updateSize();
     void setRootObject(QObject *);
@@ -108,37 +108,8 @@ public:
     QQuickView::ResizeMode resizeMode;
     QSize initialSize;
     QElapsedTimer frameTimer;
-    QV4::PersistentValue rootItemMarker;
 };
 
-namespace QV4 {
-namespace Heap {
-
-struct QQuickRootItemMarker : Object {
-    inline QQuickRootItemMarker(QQuickWindow *window)
-        : window(window)
-    {
-    }
-
-    QQuickWindow *window;
-};
-
-}
-
-struct QQuickRootItemMarker : public Object
-{
-    V4_OBJECT2(QQuickRootItemMarker, Object)
-
-    static Heap::QQuickRootItemMarker *create(QQmlEngine *engine, QQuickWindow *window);
-
-    static void markObjects(QV4::Heap::Base *that, QV4::ExecutionEngine *e);
-
-};
-
-
-
-}
-
 QT_END_NAMESPACE
 
 #endif // QQUICKVIEW_P_H
